/*
 * carddeck is an implementation of a deck of cards.
 *
 * Copyright (C) 2008 Thomas Repantis <trep@cs.ucr.edu>
 * 
 * This file is part of carddeck.
 *
 * carddeck is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * carddeck is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with carddeck.  If not, see <http://www.gnu.org/licenses/>.
 */

//
//! \author Thomas Repantis
//! \file Deck.cc
//! Carddeck is an implementation of a deck of cards.
//

#include "Deck.h"

using namespace deck;

//! Deck constructor.
Deck::Deck(enum DeckType type) throw(error::Exception) : type_(type) {
  if (type != STANDARD) {
    throw error::Exception("Invalid deck type.");
  }

  reset();
}

//! Deck destructor.
Deck::~Deck() {
  clear_();
  // Set the size to zero.
  std::vector<Card *>().swap(cards_);
}

//!< View the cards of the deck.
std::string Deck::view() const {
  std::string allCards = "\n";

  // Since we deal the last card, we print the cards from last to
  // first.
  std::vector<Card *>::const_reverse_iterator pNextCardP;
  pNextCardP = cards_.rbegin();
  while (pNextCardP != cards_.rend()) {
    allCards += ((*pNextCardP)->view() + "\n");
    pNextCardP++;
  }

  return allCards;
}

//!< Shuffle the remaining cards of the deck.
void Deck::shuffle() {
  // If repeating the same numbers between executions we could 
  // also use STL's random_shuffle and avoid skewness.
  time_t secs;
  time(&secs);
  srandom((unsigned int) secs);
  
  for (unsigned int i = 0; i < cards_.size(); i++) {
    int randomPosition = (int)(((double)random() / RAND_MAX) * (cards_.size()));
    Card *tmp = cards_[i];
    cards_[i] = cards_[randomPosition];
    cards_[randomPosition] = tmp;
  }
}

//!< Deal the top card of the deck.
Card *Deck::deal() throw(error::Exception) {
  if (cards_.empty()) {
    throw error::Exception("Deck is empty.");
  }

  Card *topCard = cards_.back();
  cards_.pop_back();
  return topCard;
}

//!< Recreate the deck.
void Deck::reset() {
  clear_();
  
  // Provision for different sizes of decks.
  cards_.reserve(STANDARD_CARDS_NUM_);

  createCards_(Card::CLUB);
  createCards_(Card::DIAMOND);
  createCards_(Card::HEART);
  createCards_(Card::SPADE);

  shuffle();

}

//!< Empty the deck.
void Deck::clear_() {
  while (!cards_.empty()) {
    delete cards_.back();
    cards_.pop_back();
  }
  cards_.clear(); // Superfluous.
}

//!< Create the 13 cards of a suit of a standard 52-card deck.
void Deck::createCards_(enum Card::SuitType suit) {
  for (int i = 2; i <=10 ; i++) {
    cards_.push_back(numberCardFactory_.createCard(i, suit));
  }
  cards_.push_back(letterCardFactory_.createCard(1, suit));
  cards_.push_back(letterCardFactory_.createCard(11, suit));
  cards_.push_back(letterCardFactory_.createCard(12, suit));
  cards_.push_back(letterCardFactory_.createCard(13, suit));
}
