/*
 * carddeck is an implementation of a deck of cards.
 *
 * Copyright (C) 2008 Thomas Repantis <trep@cs.ucr.edu>
 * 
 * This file is part of carddeck.
 *
 * carddeck is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * carddeck is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with carddeck.  If not, see <http://www.gnu.org/licenses/>.
 */

//
//! \author Thomas Repantis
//! \file Deck.h
//! Carddeck is an implementation of a deck of cards.
//

#ifndef __DECK_H__
#define __DECK_H__

#include <iostream>
#include <vector>
#include <string>
#include <algorithm>
#include "Exception.h"
#include "Card.h"
#include "NumberCard.h"
#include "LetterCard.h"
#include "CardFactory.h"

namespace deck {

  //! Deck is a deck of cards.
  class Deck {
  public:
    //! Types of decks, standard 52-card deck is supported.
    enum DeckType { 
      STANDARD = 1 
    };

    Deck(enum DeckType = STANDARD) throw(error::Exception);
    //!< Deck constructor.
    
    virtual ~Deck();
    //!< Deck destructor.
    
    Card *deal() throw(error::Exception);
    //!< Deal the top card of the deck.

    void shuffle();
    //!< Shuffle the remaining cards of the deck.

    void reset();
    //!< Recreate the deck.

    std::string view() const;
    //!< View the cards of the deck.

  private:
    //!< Empty the deck.
    void clear_();

    //!< Create the 13 cards of a suit of a standard 52-card deck.
    void createCards_(enum Card::SuitType suit);

    std::vector<Card *> cards_;
    //!< The deck of cards.
    // Storing the cards in a vector gives constant time access of random 
    // elements for shuffling, and constant amortized time removal of the 
    // last element for dealing.  

    enum DeckType type_;
    //!< The type of the deck, e.g. standard 52-card deck.

    CardFactory<NumberCard> numberCardFactory_;
    //!< The factory producing number cards.
    
    CardFactory<LetterCard> letterCardFactory_;
    //!< The factory producing letter cards.

    //! The number of cards in a standard deck.
    static const int STANDARD_CARDS_NUM_ = 52;
  };

  inline std::ostream &operator<<(std::ostream &out, const Deck &deck) {
    return out << deck.view();
  }
  //!< Print the names of all cards of the deck.
  
}

#endif
